#!/usr/bin/env python3.4

import os
import re
import sys
import argparse

parser = argparse.ArgumentParser(description='Prepare kernel files for Raspberry Pi bootloader')
parser.add_argument('--force', '-f', action='store_true', default=False,
  help='overwrite target file')
parser.add_argument('kernel',
  help='kernel file from /usr/src/linux*/arch/arm/boot/Image')
parser.add_argument('bootimage',
  help='file to be placed in /boot/kernel.img')
args = parser.parse_args()
if os.path.exists(args.bootimage) and not args.force:
  print('Target file exists, use --force to override')
  sys.exit(1)

f = open(args.kernel, 'rb')
kernel_image = f.read()
f.close()
   
re_line = re.compile(r"0x(?P<value>[0-9a-f]{8})")

mem = [0 for i in range(32768)]

def load_to_mem(name, addr):
   f = open(name)

   for l in f.readlines():
      m = re_line.match(l)

      if m:
         value = int(m.group("value"), 16)

         for i in range(4):
            mem[addr] = int(value >> i * 8 & 0xff)
            addr += 1

   f.close()

load_to_mem("/usr/share/raspberrypi-mkimage/boot-uncompressed.txt", 0x00000000)
load_to_mem("/usr/share/raspberrypi-mkimage/args-uncompressed.txt", 0x00000100)

f = open(args.bootimage, "wb")

for m in mem:
   f.write(chr(m))

f.write(kernel_image)
f.close()
